/*
 *
 *  $Id: visualizatorcontroller.cpp 4706 2012-02-16 07:59:43Z tovar $
 *  Ginkgo CADx Project
 *
 *  Copyright 2008-10 MetaEmotion S.L. All rights reserved.
 *  http://ginkgo-cadx.com
 *
 *  This file is licensed under LGPL v3 license.
 *  See License.txt for details
 *
 *
 */
#include <wx/wx.h>
#include <wx/notebook.h>
#include <wx/dir.h>
#include <wx/file.h>
#include <set>
#include "visualizatorcontroller.h"

#include "../herramientas/herramientamapacolor.h"
#include "../herramientas/herramientaoverlays.h"
#include "../herramientas/reconstructiontool.h"

#include <visualizator/dialogos/contenedorherramientas.h>
#include <main/gui/import/generaldata.h>
#include <main/gui/import/importationdata.h>
#include <visualizator/dialogos/seleccionardiagnostico/seleccionardiagnostico.h>
#include "../vistas/vista2d.h"
#include "../vistas/waveformview.h"
#include <visualizator/estudios/visualizatorstudy.h>
#include <visualizator/license.h>

#include <api/ientorno.h>
#include <api/ivista.h>
#include <api/dicom/imodelodicom.h>
#include <api/integration/dict.h>
#include <api/icontroladorcarga.h>
#include <main/controllers/controladorhistorial.h>
#include <visualizator/dialogos/importacion/selectimagesimportationvisualizator.h>

#include <api/icontroladorpermisos.h>

#include <api/icontroladorherramientas.h>

#include <api/controllers/ipacscontroller.h>
#include <api/icontroladorvistas.h>

#include <api/icontroladorcarga.h>
#include <main/controllers/controladorcarga.h>
#include <main/controllers/controladorcomandos.h>
#include <visualizator/commands/pdfextractercommand.h>
#include <commands/comandopacs.h>

#include <vector>
#include <string>
#include <sstream>
#include <iostream>

#include <api/icontroladorinternacionalizacion.h>
#include <visualizator/resources/aprimresourcemanager.h>
#include "../exportacion/tagsprivados.h"

#define UID_EncapsulatedPDFStorage  "1.2.840.10008.5.1.4.1.1.104.1"

namespace GNKVisualizator {
	class GNKVisualizatorControllerMode: public GNC::GCS::ModoControlador
	{
	public:
		//GNKVisualizatorControllerMode: defines ways of Dicom viewing of this extension
		//id: unique in this controller
		//listaModalidades: modality supporting list
		//abreSeriesSueltas: true if it opens series, false if it opens studies
		//bitmap: bitmap of this mode (for visualization purposes)
		GNKVisualizatorControllerMode(int id, const std::string& descripcion, TipoListaModalidades listaModalidades, TipoListaUIDsImportacion listaUIDsImportacion, bool abreSeriesSueltas, wxBitmap * bitmap = NULL):GNC::GCS::ModoControlador(id, descripcion, listaModalidades, listaUIDsImportacion, abreSeriesSueltas, bitmap)
		{
		}
		~GNKVisualizatorControllerMode() {}

		//returns true if this mode supports this pair of modality-transfer syntax
		virtual bool SupportsModalityFile(const std::string &modalidad, const std::string& transferSyntax){
			for(TipoListaModalidades::iterator it = m_listaModalidades.begin(); it!= m_listaModalidades.end(); ++it){
				if((*it) == modalidad && !(transferSyntax == "1.2.840.10008.1.2.4.100" || transferSyntax == "1.2.840.10008.1.2.4.101")){
					return true;
				}
			}
			return false;
		}
	};
}


/* Constructor */
GNKVisualizator::ControladorAtencionPrimaria::ControladorAtencionPrimaria(GNC::GCS::IEntorno* pEntorno) : GNC::GCS::IControladorModulo(pEntorno, EXT_SID, 0)
{
	//primero de todo registramos nuestro catalogo para la internacionalizacion de las cadenas
	{
		pEntorno->GetControladorInternacionalizacion()->AddCatalog("visualizator");
	}

	//registramos los permisos
	m_pEntorno->GetControladorPermisos()->AddDefinicionPermiso("atencionprimaria",_Std("Ginkgo CADx viewer"), "dicomizador",_Std("Simple importation of images"),false, "", true);
	m_pEntorno->GetControladorPermisos()->AddDefinicionPermiso("atencionprimaria",_Std("Ginkgo CADx viewer"),"vista2d",_Std("Medical viewer"), false, "", true);
	m_pEntorno->GetControladorPermisos()->AddDefinicionPermiso("atencionprimaria",_Std("Ginkgo CADx viewer"),"waveform",_Std("Waveform viewer"), false, "", true);
	m_pEntorno->GetControladorPermisos()->AddDefinicionPermiso("atencionprimaria",_Std("Ginkgo CADx viewer"),"reconstruction",_Std("3D reconstruction"), false, "", true);

	m_pEntorno->GetControladorPermisos()->AddDefinicionPermiso("atencionprimaria.reconstruction",_Std("3D reconstruction"),"orthogonalmpr3d",_Std("3D orthogonal multi-planar reconstruction (MPR)"), false, "", false);
	m_pEntorno->GetControladorPermisos()->AddDefinicionPermiso("atencionprimaria.reconstruction",_Std("3D reconstruction"),"mip3d",_Std("3D maximun intensity projection (MIP)"), false, "", false);
	m_pEntorno->GetControladorPermisos()->AddDefinicionPermiso("atencionprimaria.reconstruction",_Std("3D reconstruction"),"volumerengering3d",_Std("3D volume rendering"), false, "", true);
	m_pEntorno->GetControladorPermisos()->AddDefinicionPermiso("atencionprimaria.reconstruction",_Std("3D reconstruction"),"surfacerendering3d",_Std("3D surface rendering"), false, "", true);
	m_pEntorno->GetControladorPermisos()->AddDefinicionPermiso("atencionprimaria.reconstruction",_Std("3D reconstruction"),"endoscopy3d",_Std("3D endoscopy"), false, "", false);

	m_pEntorno->GetControladorPermisos()->AddDefinicionPermiso("atencionprimaria.vista2d",_Std("Medical viewer"),"guardar cambios",_Std("Save changes with medical viewer"), true, "", true);


	m_pEntorno = pEntorno;	
	m_ImporterDescription = _Std("Default importer");
	
	// Codigos LOINC de los informes de A. Primaria.
	m_ListaInformes.push_back(GNC::GCS::IdHL7("TUID" + m_UID, "0", _Std("Ginkgo CADx Image Report"), "LN", "", "","v1.0"));

	m_pPanelContenedorCustom = NULL;

	//registrar herramientas
	GNC::GCS::IControladorHerramientas* cH = m_pEntorno->GetControladorHerramientas();
	cH->RegistrarHerramienta(new GNKVisualizator::HerramientaMapaColor());
	cH->RegistrarHerramienta(new GNKVisualizator::HerramientaOverlays());
	cH->RegistrarHerramienta(new GNKVisualizator::ReconstructionTool(m_pEntorno->GetControladorPermisos()));

}

GNKVisualizator::ControladorAtencionPrimaria::~ControladorAtencionPrimaria()
{
	//desregistrar herramienta
	GNKVisualizator::IHerramientaMapaColor* hMapa;
	GNKVisualizator::IHerramientaOverlays*  hOverlays;
	GNKVisualizator::IReconstructionTool*   hReconstruction;
	GNC::GCS::IControladorHerramientas* cH = m_pEntorno->GetControladorHerramientas();
	try {
		hMapa = cH->ObtenerHerramientaConcreta<GNKVisualizator::IHerramientaMapaColor>(GNKVisualizator::IHerramientaMapaColor::ID);
		if (hMapa != NULL) {
			cH->DesRegistrarHerramienta(hMapa);
			delete hMapa;
		}
		hOverlays = cH->ObtenerHerramientaConcreta<GNKVisualizator::IHerramientaOverlays>(GNKVisualizator::IHerramientaOverlays::ID);
		if (hOverlays != NULL) {
			cH->DesRegistrarHerramienta(hOverlays);
			delete hOverlays;
		}
		hReconstruction = cH->ObtenerHerramientaConcreta<GNKVisualizator::IReconstructionTool>(GNKVisualizator::IReconstructionTool::ID);
		if (hReconstruction != NULL) {
			cH->DesRegistrarHerramienta(hReconstruction);
			delete hReconstruction;
		}
	}
	catch (GNC::GCS::ControladorHerramientasException& ex) {
		std::cerr << "Error al registrar herramientas de Oftal: No se pudo subscribir la herramienta: " << ex << std::endl;
	}


	unsigned int i;
	for (i = 0; i < m_ListaModos.size(); ++i) {
		delete m_ListaModos[i];
	}
	m_ListaModos.clear();

	EliminarPaneles();
	if (m_pPanelContenedorCustom != NULL) {
		delete m_pPanelContenedorCustom;
		m_pPanelContenedorCustom = NULL;
	}
	#if defined(_GINKGO_DEBUG) && defined(_WINDOWS)
	//cairo_debug_reset_static_data();
	//_CrtDumpMemoryLeaks();
	#endif
}

void GNKVisualizator::ControladorAtencionPrimaria::RegistrarVistas()
{
	int idModo = 0;
	GNC::GCS::ModoControlador::TipoListaModalidades listaModalidades;
	//TODO meter las modalidades que soporta...
	listaModalidades.push_back(std::string("MR")); //resonancia
	listaModalidades.push_back(std::string("CR")); //computed radiography
	listaModalidades.push_back(std::string("US")); //ultrasonido
	listaModalidades.push_back(std::string("BI")); //biomagnetic image
	listaModalidades.push_back(std::string("CT")); //computed tomography
	listaModalidades.push_back(std::string("XA")); //XRAY angiografia
	listaModalidades.push_back(std::string("SC")); //secondary capture
	listaModalidades.push_back(std::string("NM")); //nuclear medicine
	listaModalidades.push_back(std::string("CD")); //Color flow doppler
	listaModalidades.push_back(std::string("DD")); //Duplex Doppler
	listaModalidades.push_back(std::string("DG")); //Diaphanography
	listaModalidades.push_back(std::string("ES")); //Endoscopy
	listaModalidades.push_back(std::string("LS")); //Laser surface scan
	listaModalidades.push_back(std::string("PT")); //Positron emission tomography (PET)
	listaModalidades.push_back(std::string("RG")); //Radiographic imaging
	listaModalidades.push_back(std::string("ST")); //Single-photon emission computed tomography (SPECT)
	listaModalidades.push_back(std::string("MG")); //Mammography
	listaModalidades.push_back(std::string("IO")); //Intra-Oral
	listaModalidades.push_back(std::string("TG")); //Thermography
	listaModalidades.push_back(std::string("RF")); //Radio Fluoroscopy
	listaModalidades.push_back(std::string("RTIMAGE")); //Radiotherapy Image
	listaModalidades.push_back(std::string("RTDOSE")); //Radiotherapy Dose
	listaModalidades.push_back(std::string("RTSTRUCT")); //Radiotherapy Structure Set
	listaModalidades.push_back(std::string("RTPLAN")); //Radiotherapy Plan
	listaModalidades.push_back(std::string("RTRECORD")); //RT Treatment Record
	listaModalidades.push_back(std::string("HC")); //Hard Copy
	listaModalidades.push_back(std::string("DX")); //Digital Radiography
	listaModalidades.push_back(std::string("PX")); //Panoramic X-Ray
	listaModalidades.push_back(std::string("GM")); //General Microscopy
	listaModalidades.push_back(std::string("SM")); //Slide Microscopy
	listaModalidades.push_back(std::string("XC")); //External-camera Photography
	listaModalidades.push_back(std::string("PR")); //Presentation State
	//listaModalidades.push_back(std::string("SR")); //SR Document
	listaModalidades.push_back(std::string("IVUS")); //Intravascular Ultrasound
	listaModalidades.push_back(std::string("OP")); //Stereometric Relationship
	listaModalidades.push_back(std::string("OT")); //Other
	listaModalidades.push_back(std::string("SMR")); //Intra-oral Radiography
	listaModalidades.push_back(std::string("OCT")); //Optical Coherence Tomography
	listaModalidades.push_back(std::string("OT"));  //Other
	listaModalidades.push_back(std::string("OPR")); //Ophthalmic Refraction
	listaModalidades.push_back(std::string("OPV")); //Ophthalmic Visual Field
	listaModalidades.push_back(std::string("OPM")); //Ophthalmic Mapping
	listaModalidades.push_back(std::string("KO"));  //Key Object Selection
	listaModalidades.push_back(std::string("SEG")); //Segmentation
	listaModalidades.push_back(std::string("REG")); //Registration
	listaModalidades.push_back(std::string("OPT")); //Ophthalmic Tomography
	listaModalidades.push_back(std::string("BDUS"));//Bone Densitometry (ultrasound)
	listaModalidades.push_back(std::string("BMD")); //Bone Densitometry (X-Ray)

	GNC::GCS::ModoControlador::TipoListaUIDsImportacion listaUIDsImportacion;
	listaUIDsImportacion.push_back(EXT_IMPORTER_SID);

	if(m_pEntorno->GetControladorPermisos()->Get("atencionprimaria", "vista2d")) {
		m_ListaModos.push_back(new GNKVisualizatorControllerMode(idModo++, _Std("Medical Image Viewer"), listaModalidades, listaUIDsImportacion, true));
	}

	listaModalidades.clear();
	listaModalidades.push_back(std::string("ECG")); //Electrocardiography
	listaModalidades.push_back(std::string("HD")); //Hemodynamic Waveform

	listaUIDsImportacion.clear();
	if(m_pEntorno->GetControladorPermisos()->Get("atencionprimaria", "waveform")) {
		m_ListaModos.push_back(new GNC::GCS::ModoControlador(idModo++, _Std("Waveform viewer"), listaModalidades, listaUIDsImportacion, true));
	}
}


bool GNKVisualizator::ControladorAtencionPrimaria::SupportsImportationUID(const std::string &importationUID)
{
	return (importationUID == EXT_IMPORTER_SID);
}

/* Abre un estudio de manera no interactiva desde un directorio y devuelve su controlador específico */
GNC::GCS::IVista* GNKVisualizator::ControladorAtencionPrimaria::AbrirVista(int modo, ListaDescriptoresSerie& series, const std::string& uidEstudioDiagnostico)
{
	GNC::GCS::IVista* pVista = NULL;

	// TODO: El workflow de la vista PASI es el OPTIMO AHORA.
	if (modo < 0 || modo >= (int) m_ListaModos.size()) {
		//TODO: Excepción
		return NULL;
	}

	//si no se soporta alguna modalidad pues se ignora
	std::set<std::string> uidsSeries;
	GnkPtr<GIL::IModeloIntegracion> pModeloIntegracion;
	{
		for(ListaDescriptoresSerie::iterator it = series.begin(); it!= series.end(); it++){
			if(m_ListaModos[modo]->SupportsModalityFile((*it).m_modalidad, (*it).m_uidTransferSyntax))
			{
				uidsSeries.insert((*it).m_uidSerie);
			}
		}
	}

	for(std::set<std::string>::iterator it = uidsSeries.begin(); it!= uidsSeries.end(); it++)
	{
		std::vector<std::string> rutas = m_pEntorno->GetControladorHistorial()->GetPathsSerieOrdenados((*it));
		if (rutas.size() == 0)
		{
			return NULL;
		}

		//if it's pdf...
		GNC::GCS::ControladorHistorial::ListaModelosDCM dcmModels;
		GNC::GCS::ControladorHistorial::Instance()->GetAllModelosDCMSeries((*it), dcmModels);
		if (dcmModels.size() > 0 && dcmModels.front().m_sopClassUID == UID_EncapsulatedPDFStorage) 
		{			
			GNKVisualizator::GADAPI::PDFExtracterCommandParameters* pParams = new GNKVisualizator::GADAPI::PDFExtracterCommandParameters(rutas);
			GNKVisualizator::GADAPI::PDFExtracterCommand* pCmd = new GNKVisualizator::GADAPI::PDFExtracterCommand(pParams);
			m_pEntorno->GetControladorComandos()->ProcessAsync("Extracting...", pCmd, NULL);
			
			continue;
		}

		GNC::GCS::IControladorVistas* pCtrlVistas = m_pEntorno->GetControladorVistas();
		wxWindow* pVentanaPadre = NULL;

		//primero de todo seleccionar diagnostico
		std::string uidDiagnosticoSeleccionado = uidEstudioDiagnostico;
		if(!SeleccionarDiagnostico(rutas[0],uidDiagnosticoSeleccionado))
		{
			//no quiere continuar, se cancela la carga
			return NULL;
		}

		{//comienza la deshabilitacion
			pVentanaPadre = pCtrlVistas->GetRootWindow();
			pCtrlVistas->Freeze();
			try{
				if (m_ListaModos[modo]->GetDescripcion() == _Std("Medical Image Viewer")) {
					wxWindowDisabler disabler;
					InsertarPaneles();
					GnkPtr<GNKVisualizator::VisualizatorStudy> estudio = new GNKVisualizator::VisualizatorStudy();
					estudio->ListaUIDsSerie.push_back((*it));
					estudio->Entorno = m_pEntorno;
					estudio->Modulo = this;
					estudio->VentanaPadre = pVentanaPadre;
					estudio->InicializarContextoEstudio(rutas, uidDiagnosticoSeleccionado, GNC::GCS::IContextoEstudio::TMF_UNDiagnostico);
					pVista = new GNKVisualizator::Vista2D(estudio);
					pCtrlVistas->Registrar(pVista);
				} else if (m_ListaModos[modo]->GetDescripcion() == _Std("Waveform viewer")) {
					wxWindowDisabler disabler;
					InsertarPaneles();
					GnkPtr<GNKVisualizator::ECGStudy> estudio = new GNKVisualizator::ECGStudy();
					estudio->ListaUIDsSerie.push_back((*it));
					estudio->Entorno = m_pEntorno;
					estudio->Modulo = this;
					estudio->VentanaPadre = pVentanaPadre;
					estudio->InicializarContextoEstudio(rutas, uidDiagnosticoSeleccionado, GNC::GCS::IContextoEstudio::TMF_UNDiagnostico);
					pVista = new GNKVisualizator::WaveFormView(estudio);
					pCtrlVistas->Registrar(pVista);
				}
			}
			catch(GNC::GCS::VistaException &ex){
				if (pVista != NULL) {
					pCtrlVistas->Destruir(pVista);
				}
				pCtrlVistas->Thaw();
				wxString mensaje = wxString::FromUTF8(ex.GetCause().c_str());
				wxMessageDialog dialogo( m_pEntorno->GetVentanaRaiz(), mensaje, _("Error opening study"),  wxOK | wxICON_INFORMATION);
				dialogo.ShowModal();
				return NULL;
			}
			catch (...) {
				if (pVista != NULL) {
					pCtrlVistas->Destruir(pVista);
				}
				pCtrlVistas->Thaw();
				wxMessageDialog dialogo( m_pEntorno->GetVentanaRaiz(), _("Error opening study"), _("Error opening study"),  wxOK | wxICON_INFORMATION);
				dialogo.ShowModal();
				return NULL;
			}
			pCtrlVistas->Thaw();
		}//fin deshabilitacion

		if (pVista == NULL ) {
			pCtrlVistas->Freeze();
			EliminarPaneles();
			pCtrlVistas->Destruir(pVista);
			pCtrlVistas->Thaw();
			return NULL;
		}
		else {
			m_pEntorno->GetControladorCarga()->CargaAsincrona(pVista, pModeloIntegracion, uidDiagnosticoSeleccionado);
		}
	}

	return pVista;
}


// Selecciona el diagnostico. Metodo sincrono con la interfaz.
bool GNKVisualizator::ControladorAtencionPrimaria::SeleccionarDiagnostico(const std::string& rutaFichero, std::string& uidEstudioDiagnostico)
{
	bool continuar = true;
	if(uidEstudioDiagnostico == "")
	{
		GNKVisualizator::GUI::SeleccionarDiagnostico dialogSeleccionarDiagnostico(m_pEntorno->GetVentanaRaiz(),rutaFichero,m_pEntorno);
		if(dialogSeleccionarDiagnostico.Mostrar()) {
			switch(dialogSeleccionarDiagnostico.ShowModal())
			{
				case wxID_OK:
					uidEstudioDiagnostico = dialogSeleccionarDiagnostico.GetUIDEstudioDiagnostico();
					break;
				case wxID_CANCEL:
					continuar = false;
					break;
			}
		}
	}
	return continuar;
}


bool GNKVisualizator::ControladorAtencionPrimaria::SoportaImportacion() const
{
	return m_pEntorno->GetControladorPermisos()->Get("atencionprimaria", "dicomizador");
}

void GNKVisualizator::ControladorAtencionPrimaria::GetPasosImportacion(IWizard* pWizard,std::list<IPasoWizard*> &listaPasos, wxWindow* pParent, std::string &dirTemporal, GnkPtr<GIL::IModeloIntegracion> pModeloIntegracion)
{
	GnkPtr<GNC::GUI::ImportationData> datosPersistentes(new GNC::GUI::ImportationData(pModeloIntegracion));

	IPasoWizard* paso=new GNC::GUI::GeneralData(pParent,pWizard,datosPersistentes, true);
	listaPasos.push_back(paso);

	paso = new GNKVisualizator::GUI::SelectImagesImportationVisualizator(pParent,pWizard,datosPersistentes,dirTemporal);
	listaPasos.push_back(paso);				
}

//region "Manipulacion de paneles extra (Herramientas, etc.)"

void GNKVisualizator::ControladorAtencionPrimaria::InsertarPaneles()
{
	if (m_pPanelContenedorCustom == NULL) {
		m_pPanelContenedorCustom = new GNKVisualizator::GUI::ContenedorHerramientas(m_pEntorno->GetVentanaRaiz());
		m_pPanelContenedorCustom->Freeze();
		m_pPanelContenedorCustom->Show(false);
		m_pEntorno->GetControladorVistas()->InsertarPanelIzquierdo(m_pPanelContenedorCustom, _Std("DICOM Tags").c_str());
		m_pPanelContenedorCustom->Layout();
		m_pPanelContenedorCustom->ActualizarVirtualSize();
		m_pPanelContenedorCustom->Thaw();
	}
}

void GNKVisualizator::ControladorAtencionPrimaria::EliminarPaneles()
{
	m_pEntorno->GetControladorVistas()->EliminarPanel(m_pPanelContenedorCustom);
	m_pPanelContenedorCustom=NULL;
}

void GNKVisualizator::ControladorAtencionPrimaria::MostrarPaneles(bool izquierdo , bool derecho )
{
	if (m_pPanelContenedorCustom != NULL && izquierdo) {
		m_pPanelContenedorCustom->Freeze();
		m_pEntorno->GetControladorVistas()->MostrarPanel(m_pPanelContenedorCustom);
		m_pPanelContenedorCustom->Layout();
		m_pPanelContenedorCustom->ActualizarVirtualSize();
		m_pPanelContenedorCustom->Thaw();
	}
}

void GNKVisualizator::ControladorAtencionPrimaria::OcultarPaneles(bool izquierdo, bool derecho)
{
	if ( m_pPanelContenedorCustom != NULL && izquierdo) {
		m_pPanelContenedorCustom->Freeze();
		m_pEntorno->GetControladorVistas()->OcultarPanel(m_pPanelContenedorCustom);
		m_pPanelContenedorCustom->Layout();
		m_pPanelContenedorCustom->ActualizarVirtualSize();
		m_pPanelContenedorCustom->Thaw();
	}
}
//endregion

void GNKVisualizator::ControladorAtencionPrimaria::ReprocessIntegration(const std::string& uidSeries, GnkPtr<GIL::IModeloIntegracion> pIntegrationModel)
{	
	//reprocess integration flow...

	std::vector<std::string> rutas = m_pEntorno->GetControladorHistorial()->GetPathsSerieOrdenados(uidSeries);
	GnkPtr<GNKVisualizator::VisualizatorStudy> estudio = new GNKVisualizator::VisualizatorStudy();
	estudio->ListaUIDsSerie.push_back(uidSeries);
	estudio->Entorno = m_pEntorno;
	estudio->Modulo = this;
	estudio->VentanaPadre = NULL;
	estudio->InicializarContextoEstudio(rutas, "", GNC::GCS::IContextoEstudio::TMF_UNDiagnostico);

	GIL::IStudyModel studyModel;
	GIL::ISeriesModel seriesModel;
	//recreate evidences...
	std::string strTmp0, strTmp1;

	estudio->SetIndiceActivo(0);
	estudio->GetTagImagenActiva("0020|000d", strTmp0);//uid study
	studyModel.MapOfVars.InsertVariable(GKDI_STUDY_INSTANCE_UID, strTmp1);
	estudio->GetTagImagenActiva("0008|0050", strTmp1);//accession
	studyModel.MapOfVars.InsertVariable(GKDI_STUDY_ACCESSION_NUMBER, strTmp1);

	estudio->GetTagImagenActiva("0020|000e", strTmp0);//uid series
	seriesModel.MapOfVars.InsertVariable(GKDI_SERIE_INSTANCE_UID, strTmp0);
	for (int i = 0; i < estudio->GetNumeroCortes(); ++i) {
		GIL::IImageModel imageModel;
		estudio->SetIndiceActivo(i);
		estudio->GetTagImagenActiva("0008|0018", strTmp0);//sop instance
		estudio->GetTagImagenActiva("0008|0016", strTmp1);//sop class
		imageModel.MapOfVars.InsertVariable(GKDI_IMAGE_SOP_INSTANCE_UID, strTmp0);
		imageModel.MapOfVars.InsertVariable(GKDI_IMAGE_SOP_CLASS_UID, strTmp1);
		seriesModel.Images.push_back(imageModel);
	}

	studyModel.Series.push_back(seriesModel);

	pIntegrationModel->Studies.push_back(studyModel);
	//
	::GADAPI::ComandoPACSParams * pParams = new ::GADAPI::ComandoPACSParams(pIntegrationModel->GlobalVariables.GetValue(GKDI_GLOBAL_PACS_STORE_SID), rutas, pIntegrationModel);
	pParams->m_informar = false;
	::GADAPI::ComandoPACS * pCmd = new ::GADAPI::ComandoPACS(pParams);
	GNC::GCS::ControladorComandos::Instance()->ProcessAsync(_Std("Uploading..."), pCmd, NULL);	
	//
}

